@@ -420,6 +420,8 @@ from sklearn.metrics import calinski_harabasz_score
 from sklearn.naive_bayes import GaussianNB
 from sklearn import cluster
 import numpy as np
+from matplotlib import colorbar
+
 
 
 class water:
@@ -872,4 +874,165 @@ class water:
             water_mask = np.where(cluster_matrix == 1,1,np.where(invalid_mask == 1,255,0)).astype('int8')
             watermask0 = water_mask==0
             watermask1 = water_mask==1
-            return watermask1
\ No newline at end of file
+            return watermask1
+
+        @staticmethod
+        def apply_mask(array, mask, no_data_value=-9999, clear_nan=True):
+            # print("My.Inside : waterdetect.Common.py : class DWutils: def apply_mask(array, mask, no_data_value=-9999, clear_nan=True):")
+            if clear_nan:
+                mask |= np.isnan(array) | np.isinf(array)
+            return np.where(mask == True, -9999, array)
+
+
+
+        @staticmethod
+        def calc_param_limits(parameter, no_data_value=-9999,min_param_value=5,max_param_value = 20):
+            # print("My.Inside : waterquality.WaterQuality.py : class DWWaterQuality(DWWaterDetect): def calc_param_limits(self, parameter, no_data_value=-9999):")
+            valid = parameter[parameter != no_data_value]
+            min_value = np.percentile(valid, 1) if min_param_value is None else min_param_value
+            # min_value = np.quantile(valid, 0.25) if self.config.min_param_value is None else self.config.min_param_value
+            max_value = np.percentile(valid, 75) if max_param_value is None else max_param_value
+            # max_value = np.quantile(valid, 0.75) if self.config.max_param_value is None else self.config.max_param_value
+            return max_value * 1.1, min_value * 0.8
+
+        @staticmethod
+        def test():
+            print('just test')
+
+        @staticmethod
+        def gray2color_ramp(grey_array,min_value=0, max_value=20, colormap='viridis', limits=(0, 1)):
+            # print("My.Inside : waterdetect.Common.py : class DWutils: def gray2color_ramp()")
+            """
+            Convert a greyscale n-dimensional matrix into a rgb matrix, adding 3 dimensions to it for R, G, and B
+            The colors will be mixed
+            :param max_value: Maximum value for the color ramp, if None, we consider max(grey)
+            :param min_value: Minimum value for the color ramp, if None, we consider min(grey)
+            :param grey_array: greyscale vector/matrix
+            :param color1: Color for the minimum value
+            :param color2: Color for the mid value
+            :param color3: Color for the maximum value
+            :param limits: Final boundary limits for the RGB values
+            :return: Colored vector/matrix
+            """
+
+            cm = plt.get_cmap(colormap)
+            # # normaliza dentro de min e max values
+            grey_vector = (grey_array - min_value) / (max_value - min_value)
+
+            # # cut the values outside the limits of 0 and 1
+            grey_vector[grey_vector < 0] = 0
+            grey_vector[grey_vector > 1] = 1
+
+            # # Apply the colormap like a function to any array:
+            colored_image = cm(grey_vector)
+
+            return MinMaxScaler(limits).fit_transform(colored_image[:, 0:3])
+        
+
+        def reg_burn_area(self,red, green, blue, burn_in_array, color=None, min_value=None, max_value=None, colormap='viridis',
+                        fade=1, uniform_distribution=False, no_data_value=-9999, valid_value=1, transp=0.0):
+            if color:
+                new_red = np.where(burn_in_array == valid_value, color[0] * (1 - transp) + red * (transp), red * fade)
+                new_green = np.where(burn_in_array == valid_value, color[1] * (1 - transp) + green * (transp), green * fade)
+                new_blue = np.where(burn_in_array == valid_value, color[2] * (1 - transp) + blue * (transp), blue * fade)
+            else:
+                # the mask is where the value equals no_data_value
+                mask = (burn_in_array == no_data_value)
+                
+                # the valid values are those outside the mask (~mask)
+                burn_in_values = burn_in_array[~mask]
+
+                # apply scalers to uniform the data
+                if uniform_distribution:
+                    burn_in_values = QuantileTransformer().fit_transform(burn_in_values[:, np.newaxis])[:, 0]
+
+                rgb_burn_in_values = self.gray2color_ramp(burn_in_values, min_value=min_value, max_value=max_value,
+                                                    colormap=colormap, limits=(0, 1.))
+                
+                # return the scaled values to the burn_in_array
+                burn_in_array[~mask] = rgb_burn_in_values[:, 0]
+                burn_in_red = np.copy(burn_in_array)
+
+                burn_in_array[~mask] = rgb_burn_in_values[:, 1]
+                burn_in_green = np.copy(burn_in_array)
+
+                burn_in_array[~mask] = rgb_burn_in_values[:, 2]
+                burn_in_blue = np.copy(burn_in_array)
+
+                # burn in the values
+                new_red = np.where(burn_in_array == no_data_value, red*fade, burn_in_red)
+                new_green = np.where(burn_in_array == no_data_value, green*fade, burn_in_green)
+                new_blue = np.where(burn_in_array == no_data_value, blue*fade, burn_in_blue)
+                
+            return new_red, new_green, new_blue   
+
+        @staticmethod
+        def waterquality_fn(Red=1):
+            a = 493.65
+            b = 1.16
+            c = 0.188
+            tsm = a * Red / (1- (Red/c)) + b
+            return tsm 
+
+        def waterquality(self,watermask,waterquality_function=None,range=None,colormap=True,bright=5,title='',label=''):
+            # Watermask
+            self.watermask = watermask
+
+            # Reference band , Prepare parameter
+            if waterquality_function == None:
+                parameter = self.waterquality_fn(self.bands['Red'])
+            else: 
+                parameter = waterquality_function(self.bands['Red'])  # fix : bands that the same with input function
+            parameter = self.apply_mask(parameter,~watermask,-9999)
+            
+            # Min Max range value
+            if range == None:
+                self.max_value,self.min_value = self.calc_param_limits(parameter)
+            else:
+                self.min_value = range[0]
+                self.max_value = range[1]
+
+
+            # Color map 
+            if colormap != True:
+                cmap = plt.get_cmap(colormap)
+            else:
+                cmap = objearth.bluesea()
+
+            # Color bar
+            fig = plt.figure(figsize=(4,1))
+            ax1 = fig.add_axes([0.05,0.50,2.60,0.35])
+            norm = colors.Normalize(vmin=self.min_value,vmax=self.max_value)
+            cb1 = colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm,orientation='horizontal')
+            ax1.set_title(title)
+            cb1.set_label(label)
+
+            # Prepare RGB bands
+            self.bright = bright
+            red    = self.bands['Red']*self.bright
+            green  = self.bands['Green']*self.bright
+            blue   = self.bands['Blue']*self.bright
+            red[red>1]     = 1
+            green[green>1] = 1
+            blue[blue>1]   = 1
+
+            # Apply function on the area
+            red,green,blue = self.reg_burn_area(red=red,
+                                green=green,
+                                blue=blue,
+                                burn_in_array=parameter,
+                                color=None,
+                                fade=1,
+                                min_value=self.min_value,
+                                max_value=self.max_value,
+                                colormap=cmap,
+                                uniform_distribution=False,
+                                no_data_value=-9999)
+
+            # plotshow Result
+            rgb = np.stack([red,green,blue],axis=2)
+            plt.figure(figsize=(14,14))
+            plt.axis('off')
+            plt.imshow(rgb)
+            
+            return red*10000,green*10000,blue*10000
